"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DotenvPlugin = void 0;
const fs_1 = require("fs");
const dotenv_defaults_1 = __importDefault(require("dotenv-defaults"));
const core_1 = require("@rspack/core");
const utils_1 = require("./utils");
class DotenvPlugin {
    constructor(options = {}) {
        this.config = Object.assign({ path: './.env', prefix: 'process.env.' }, options);
    }
    apply(compiler) {
        var _a;
        const variables = this.gatherVariables();
        const target = (_a = compiler.options.target) !== null && _a !== void 0 ? _a : 'web';
        const data = this.formatData(variables, target);
        new core_1.DefinePlugin(Object.assign({}, data)).apply(compiler);
    }
    gatherVariables() {
        const { safe, allowEmptyValues } = this.config;
        const vars = this.initializeVars();
        const { env, blueprint } = this.getEnvs();
        Object.keys(blueprint).forEach(key => {
            const value = Object.prototype.hasOwnProperty.call(vars, key)
                ? vars[key]
                : env[key];
            const isMissing = typeof value === 'undefined' ||
                value === null ||
                (!allowEmptyValues && value === '');
            if (safe && isMissing) {
                throw new Error(`Missing environment variable: ${key}`);
            }
            else {
                vars[key] = value;
            }
        });
        // add the leftovers
        if (safe) {
            Object.keys(env).forEach(key => {
                if (!Object.prototype.hasOwnProperty.call(vars, key)) {
                    vars[key] = env[key];
                }
            });
        }
        return vars;
    }
    initializeVars() {
        return this.config.systemvars ? Object.assign({}, process.env) : {};
    }
    getEnvs() {
        const { path, silent, safe } = this.config;
        const env = dotenv_defaults_1.default.parse(this.loadFile(path, silent), this.getDefaults());
        let blueprint = env;
        if (safe) {
            let file = `${path}.example`;
            if (safe !== true) {
                file = safe;
            }
            blueprint = dotenv_defaults_1.default.parse(this.loadFile(file, silent));
        }
        return {
            env,
            blueprint,
        };
    }
    getDefaults() {
        const { path, defaults, silent } = this.config;
        if (defaults) {
            return this.loadFile(defaults === true ? `${path}.defaults` : defaults, silent);
        }
        return '';
    }
    formatData(variables, target) {
        const { expand, prefix } = this.config;
        const formatted = {};
        for (const [key, value] of Object.entries(variables)) {
            formatted[`${prefix}${key}`] = JSON.stringify(expand && typeof value === 'string'
                ? (0, utils_1.interpolate)(value, variables)
                : value);
        }
        // We have to stub any remaining `process.env`s due to Webpack 5 not polyfilling it anymore
        // https://github.com/mrsteele/dotenv-webpack/issues/240#issuecomment-710231534
        // However, if someone targets Node or Electron `process.env` still exists, and should therefore be kept
        // https://webpack.js.org/configuration/target
        if (this.shouldStub(target)) {
            // Results in `"MISSING_ENV_VAR".NAME` which is valid JS
            formatted['process.env'] = '"MISSING_ENV_VAR"';
        }
        return formatted;
    }
    shouldStub(targetInput) {
        const targets = Array.isArray(targetInput) ? targetInput : [targetInput];
        return targets.every(target => 
        // If configured prefix is 'process.env'
        this.config.prefix === 'process.env.' &&
            // If we're not configured to never stub
            this.config.ignoreStub !== true &&
            // And
            // We are configured to always stub
            (this.config.ignoreStub === false ||
                // Or if we should according to the target
                (typeof target === 'string' &&
                    !target.includes('node') &&
                    !(0, utils_1.isMainThreadElectron)(target))));
    }
    loadFile(file, silent) {
        try {
            return (0, fs_1.readFileSync)(file, 'utf8');
        }
        catch (err) {
            this.warn(`Failed to load ${file}.`, silent);
            return '';
        }
    }
    warn(msg, silent) {
        !silent && console.warn(msg);
    }
}
exports.DotenvPlugin = DotenvPlugin;
